--[[
-- TinyTip by Thrae of Maelstrom (aka "Matthew Carras")
-- 6-26-06
-- Version 1.2
-- Based very losely off AF_Tooltip_Mini
-- Some code from PerfectTooltip by cladhaire
--
-- Note: Whenever you see -- , like what started this line, this is what's called a "comment".
-- Comments are not real code, they are just there so I can communicate with you.
--]]

-- Always tries to anchor the cursor to your mouse when 2 is given, or only with units 
-- when 1 is given.
-- If this is set to nil or false, then TinyTip won't affect anchoring at ALL.
local uAnchorCursor = 2			-- Default: 2

-- This is the scale of the Tooltip. When you change the global UI Scale, it doesn't
-- scale the Tooltip, so you can do it here if you want. Leave nil if you don't care.
local uScale		= nil		-- Default: nil		Game's Default: 1.0

-- Change these values to nil to disable features, change from nil to true to enable.
local uFade					= true	-- Cool Fadeout feature for Tooltip! Default: true

local uDisplayPvPRankName	= true	-- Display the PvP Rank Name in front 
									-- of the unit name. Default: true

local uDisplayGuild			= true	-- Display the Guild Name on 
									-- the bottom line. Default: true

local uDisplayReactionText	= nil	-- Display Reaction Text 
									-- (Hostile, Friendly, etc.) Default: nil (false)

local uDisplayCivilians		= true	-- Display "Civilian" (localized) for 
									-- Civilian NPCs. Default: true

local uAlwaysColorBackdrop  = nil	-- Always color backdrop even for NPCs. 
									-- Default: nil (false)

local uColorFriends			= true	-- Color the background pink for 
									-- friends and purple for guild.

local uICanDoMath			= nil	-- Always use ?? instead of >(Your Level +10) for 
									-- unknown levels. Default: nil (false)

local uTargetOfMouse		= true	-- Add Target of Target, if it exists. Default: true

local uTargetOfParty		= 1		-- Add the names of people in your party who are currently
									-- targeting this unit. Set this to 2 if you would rather
									-- just display the number of people targeting. Default: 1
											
local uTargetOfRaid			= true	-- Add the number of people in your raid who are currently
									-- targeting this unit. Default: true

--------------------------------------------------------------------
-- Localization (German, French, English)

-- Don't change these locals here, change them below.
local locTapped
local locRareElite
local locTargeting
local locYOU
local locTargetedBy

-- Localization
-- Any wrong words, change them here.
local tmp = GetLocale()
if tmp == "deDE" then -- Deutsch, thanks Slayman
	locTapped		= "Markiert"
	locRareElite	= " Elite Rar"
	locTargeting	= "Ziel"
	locYOU			= "DU"
	locTargetedBy	= "im Ziel von"
elseif tmp == "frFR" then -- Francais
	locTapped		= "Engag\195\169"
	locRareElite	= " Elite Rare"
	locTargeting	= "Optimisation"	-- babelfish
	locYOU			= "VOUS"			-- babelfish
	locTargetedBy	= "Vis\195\168 pr\195\169s" -- babelfish
else -- English / Fallback
	locTapped		= "Tapped"
	locRareElite	= string.format("%s %s", getglobal("ITEM_QUALITY3_DESC"), getglobal("ELITE") )
	locTargeting	= "Targeting"
	locYOU			= "YOU"
	locTargetedBy	= "Targeted by"
end

--------------------------------------------------------------------
-- Local Functions

local function color_player(unit)
	_,tmp=UnitClass(unit)
	if tmp and RAID_CLASS_COLORS[tmp] then
		return string.format("%2x%2x%2x", RAID_CLASS_COLORS[tmp].r*255, 
										  RAID_CLASS_COLORS[tmp].g*255, 
										  RAID_CLASS_COLORS[tmp].b*255)
	else
		return "FFFFFF"
	end
end

local ufPreviousName
local guildonce
local function tooltip_format(unit)
	local i,treac,cdeadortap,levelLine,llText,bdead
	-- First, find Level line
	-- 
	tmp = GameTooltip:NumLines()
	for i = 1,tmp,1 do
		llText = getglobal("GameTooltipTextLeft" .. i):GetText()
		if llText then
			if string.find(llText, getglobal("LEVEL"), 1, true) then
				llText = getglobal("GameTooltipTextLeft" .. i)
				levelLine = i
				break
			end
		else
			GameTooltip:Hide()
		end
	end

	if levelLine then
		-- remove that darn PvP line
		tmp = GameTooltip:NumLines()
		for	i = levelLine+1, tmp,1 do
			tmp2 = getglobal("GameTooltipTextLeft" .. i)
			if tmp2 and tmp2:GetText() then			-- GlobalStrings.lua
				if string.find(tmp2:GetText(), getglobal("PVP_ENABLED"), 1, true) then
					tmp2:SetText(empty)
					break
				end
			end
		end
	end
	
	-- Get Rank, then Set Name (rankName, rankNumber)
	tmp,tmp2 = GetPVPRankInfo(UnitPVPRank(unit), unit)
	if uDisplayPvPRankName and UnitIsPlayer(unit) and tmp2 > 0 then
		GameTooltipTextLeft1:SetText( string.format("%s %s", tmp, UnitName(unit) or "Unknown" ) )
	else
		GameTooltipTextLeft1:SetText( UnitName(unit) or "Unknown" )
	end

	-- Reaction
	tmp,tmp2 = UnitCanAttack(unit, "player"), UnitCanAttack("player", unit)
	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		GameTooltipTextLeft1:SetTextColor(0.54,0.54,0.54)
		cdeadortap = "888888"
		if levelLine and levelLine == 3 then 
			GameTooltipTextLeft2:SetTextColor(0.54,0.54,0.54)
		end
		treac = locTapped
	elseif (tmp and tmp2) or UnitIsTappedByPlayer(unit) 
	or (tmp and not tmp2) then -- hostile
		if UnitIsTappedByPlayer(unit) then GameTooltip:SetBackdropColor(0, 0, 0) end
		GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[3].r,
										  FACTION_BAR_COLORS[3].g,
										  FACTION_BAR_COLORS[3].b)
		if tmp and not tmp2 then -- in their faction zone
			if UnitPlayerControlled(unit) or uAlwaysColorBackdrop then
				GameTooltip:SetBackdropColor(0.5, 0.2, 0.1)
			end
			treac = FACTION_STANDING_LABEL2 .. " " .. getglobal("PVP_ENABLED")
		else
			if UnitPlayerControlled(unit) or uAlwaysColorBackdrop then
				GameTooltip:SetBackdropColor(0.5, 0.0, 0.0)
			end
			treac = FACTION_STANDING_LABEL2
		end
	elseif not tmp and tmp2 then -- neutral
		if UnitPlayerControlled(unit) or uAlwaysColorBackdrop then
			GameTooltip:SetBackdropColor(0.5, 0.5, 0.0)
		end
		GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[4].r,
										  FACTION_BAR_COLORS[4].g,
										  FACTION_BAR_COLORS[4].b)
		treac = FACTION_STANDING_LABEL4
	else -- friendly
		treac = FACTION_STANDING_LABEL5
		if UnitPlayerControlled(unit) or uAlwaysColorBackdrop then
			GameTooltip:SetBackdropColor(0.0, 0.0, 0.5)
		end
		if UnitIsPVP(unit) then -- friendly, PvP-enabled
			GameTooltipTextLeft1:SetTextColor(FACTION_BAR_COLORS[6].r,
											 FACTION_BAR_COLORS[6].g,
											 FACTION_BAR_COLORS[6].b)
		else -- just friendly
			GameTooltipTextLeft1:SetTextColor(0,0.67,1)
		end
	end
	
	-- Check for a dead unit, but try to leave out Hunter's Feign Death
	if UnitHealth(unit) <= 0 and 
	( not UnitIsPlayer(unit) or ( UnitIsPlayer(unit) and UnitIsDeadOrGhost(unit) ) ) then
		GameTooltipTextLeft1:SetTextColor(0.54,0.54,0.54)
		if llText then cdeadortap = "888888" end
		if levelLine and levelLine == 3 then
			GameTooltipTextLeft2:SetTextColor(0.54,0.54,0.54)
		end
		bdead = true
	end	

	-- color line 2, if it's available
	if not cdeadortap and levelLine and levelLine == 3 then
		GameTooltipTextLeft2:SetTextColor( GameTooltipTextLeft1:GetTextColor() )
	end
	
	if levelLine then
		local clevel
		tmp2 = UnitLevel(unit) - UnitLevel("player") -- Level difficulty
		if tmp2 and UnitFactionGroup(unit) ~= UnitFactionGroup("player") then
			if tmp2 >= 5 or tmp == -1 then clevel = "FF0000"
			elseif tmp2 >= 3 then clevel = "FF6600"
			elseif tmp2 >= -2 then clevel = "FFFF00"
			elseif -tmp2 <= GetQuestGreenRange() then clevel = "00FF00"
			else clevel = "888888"
			end
		else clevel = "FFCC00"			
		end

		tmp2 = UnitLevel(unit) -- Level #
		if tmp2 and tmp2 >= 1 then 
			llText:SetText( string.format( "|cFF%s%s %d|r", 
							cdeadortap or clevel or "FFFFFF", getglobal("LEVEL"), tmp2 ) )
		elseif not uICanDoMath and tmp2 and tmp2 == -1 and tmp2 < 60 then
			llText:SetText( string.format( "|cFF%s%s >%d|r", 
							cdeadortap or clevel or "FFFFFF", getglobal("LEVEL"), tmp2 + 10 ) )
		else
			llText:SetText( string.format( "|cFF%s%s ??|r", 
							cdeadortap or clevel or "FFFFFF", getglobal("LEVEL"), tmp2 + 10 ) )
		end

		tmp2 = UnitClassification(unit) -- Elite status
		if tmp2 and tmp2 ~= "normal" and UnitHealth(unit) > 0 then
			if tmp2 == "elite" then
				llText:SetText(string.format("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFCC00",
											getglobal("ELITE") ) )
			elseif tmp2 == "worldboss" then
				llText:SetText(string.format("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FF0000",
											getglobal("BOSS") ) )
			elseif tmp2 == "rare" then
				llText:SetText(string.format("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FF66FF",
											getglobal("ITEM_QUALITY3_DESC") ) )
			elseif tmp2 == "rareelite" then
				llText:SetText(string.format("%s |cFFFFAAFF%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFAAFF",
											locRareElite ) )
			else
				llText:SetText(string.format("%s |cFF%s%s|r", 
											llText:GetText() or "", 
											cdeadortap or "FFFFFF",
											tmp2 ) )
			end
		end

		-- Class And Race
		if UnitIsPlayer(unit) then
			llText:SetText( string.format("%s |cFF%s%s|r |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							UnitRace(unit) or "Unknown", 
							cdeadortap or color_player(unit), UnitClass(unit) or "Class" ) ) 
		elseif UnitPlayerControlled(unit) then 
			llText:SetText( string.format("%s |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							UnitCreatureFamily(unit) or "Unknown" ) )
		else
			llText:SetText( string.format("%s |cFF%s%s|r",
							llText:GetText() or "",
							cdeadortap or "DDEEAA",
							UnitCreatureType(unit) or "Unknown" ) )
		end
	end
	
	-- add corpse/tapped line
	if cdeadortap and llText then 
		llText:SetText( string.format( "%s |cFF888888(%s)|r", 
						llText:GetText(), 
						( bdead and getglobal("CORPSE") ) or locTapped ) )
	end
	
	-- We like to know who our friends are.
	if uColorFriends and UnitIsPlayer(unit) and treac == FACTION_STANDING_LABEL5 then
		for i = 1,GetNumFriends(),1 do
			tmp,tmp2 = GetFriendInfo(i)
			if tmp and tmp2 ~= 0 and UnitName(unit) and tmp == UnitName(unit) then
				GameTooltip:SetBackdropColor(0.58, 0.0, 0.83)
				break
			end
		end
	end

	if not ufPreviousName or ufPreviousName ~= UnitName(unit) or UnitIsUnit(unit,"player") then
		-- Add Reaction Text and Civilian Text, if enabled
		if uDisplayReactionText and treac and uDisplayCivilians and UnitIsCivilian(unit) then
			GameTooltip:AddLine( string.format("%s %s", treac, getglobal("CIVILIAN") ),
								 GameTooltipTextLeft1:GetTextColor() )
		elseif uDisplayReactionText and treac then
			GameTooltip:AddLine( treac, GameTooltipTextLeft1:GetTextColor() )
		elseif uDisplayCivilians and UnitIsCivilian(unit) then
			GameTooltip:AddLine( getglobal("CIVILIAN"), GameTooltipTextLeft1:GetTextColor() )
		end	

		-- Add Guild info, if enabled
		tmp2 = GetGuildInfo(unit)
		if uDisplayGuild and levelLine and tmp2 then
			-- first, check to make sure we didn't already write this &^%%%@
			local guildonce
			local nLines = GameTooltip:NumLines()
			for	i = levelLine+1, nLines,1 do
				tmp = getglobal("GameTooltipTextLeft" .. i)
				if tmp and tmp:GetText() then
					if string.find(tmp:GetText(), tmp2, 1, true) then
						guildonce = true
						break
					end
				end
			end
			if not guildonce then
				tmp2 = string.format("<%s>", tmp2) 
				if IsInGuild() and tmp2 == GetGuildInfo("player") then
					if uColorFriends and not UnitIsUnit(unit,"player") then
						GameTooltip:SetBackdropColor(0.8, 0.2, 1)
						GameTooltip:AddLine( tmp2, GameTooltipTextLeft1:GetTextColor() )
					else
						GameTooltip:AddLine( tmp2, 0.14, 0.55, 0.14 )
					end
				else
					GameTooltip:AddLine( tmp2, GameTooltipTextLeft1:GetTextColor() )
				end
			end
		end

		if unit == "mouseover" then
			-- Add Target of Mouseover
			if uTargetOfMouse and UnitExists("mouseovertarget") then
				if UnitIsUnit("mouseovertarget", "player") then
					GameTooltip:AddLine( string.format("%s: |cFFFF2222%s|r", locTargeting, locYOU) )
				elseif UnitIsPlayer("mouseovertarget") then
					GameTooltip:AddLine( string.format("%s: |cFF%s%s|r", 
										 locTargeting, 
										 color_player("mouseovertarget"), 
										 UnitName("mouseovertarget") or "Unknown" ) )
				else
					GameTooltip:AddLine( string.format("%s: |cFFFFFFFF%s|r", 
										 locTargeting, 
										 UnitName("mouseovertarget") or "Unknown" ) )
				end
			end

			-- Add How Many Raid Members (#) are targeting this unit
			tmp = GetNumRaidMembers()
			if uTargetOfRaid and tmp and tmp > 0 then
				tmp2 = 0
				for i = 1,tmp,1 do
					if UnitIsUnit("raid" .. i .. "target",unit) then
						tmp2 = tmp2 + 1
					end
				end
				if tmp2 > 0 then
					GameTooltip:AddLine( string.format("%s: (|cFFFFFFFF%s|r) %s", 
														locTargetedBy, tostring(tmp2), 
														getglobal("RAID")) )
				end
			elseif uTargetOfParty then -- Add the names of the party members targeting (except you)
				local pmn = GetNumPartyMembers()
				if pmn and pmn > 0 then
					local pn
					tmp2 = nil
					for i = 1,pmn,1 do
						pid = "party" .. i	
						if UnitIsUnit(pid .. "target",unit) and not UnitIsUnit(pid, "player") then
							if uTargetOfParty ~= 2 then
								tmp2 = string.format("%s|cFF%s%s|r\n", 
													 tmp2 or "", color_player(pid), UnitName(pid) or "Unknown")
							else
								if not tmp2 then tmp2 = 0 end
								tmp2 = tmp2 + 1
							end
						end
					end
					if tmp2 then
						if uTargetOfParty ~= 2 then
							GameTooltip:AddLine( string.format("%s:\n%s", locTargetedBy, tmp2) )
						else
							GameTooltip:AddLine( string.format("%s: (|cFFFFFFFF%s|r) %s", 
												 locTargetedBy, tmp2, getglobal("TUTORIAL_TITLE19")) )
						end
					end
				end
			end
		end
		ufPreviousName = UnitName(unit)
	elseif ufPreviousName == UnitName(unit) then
		ufPreviousName = nil
	end
	
	GameTooltip:Show()
end

-- blatantly stolen from PerfectTooltip by cladhaire!
local function SmartSetOwner(owner, setX, setY, tooltip)
	if (not owner) then owner = UIParent end
	if (not tooltip) then tooltip = this end
	if (not setX) then setX = 0 end
	if (not setY) then setY = 0 end

	local x,y = owner:GetCenter()
    local left = owner:GetLeft()
	local right = owner:GetRight()
    local top = owner:GetTop()
    local bottom = owner:GetBottom()
	local screenWidth = UIParent:GetWidth()
    local screenHeight = UIParent:GetHeight()
    local scale = owner:GetScale()
    if (x~=nil and y~=nil and left~=nil and 
	right~=nil and top~=nil and bottom~=nil 
	and screenWidth>0 and screenHeight>0) then
        setX = setX * scale
        setY = setY * scale
        x = x * scale
		y = y * scale 
		left = left * scale 
		right = right * scale 
		width = right - left 
		top = top * scale 
		bottom = bottom * scale 
		local anchorPoint 
		
		if (y <= (screenHeight * (1/2))) then 
			top = top + setY 
			anchorPoint = "TOP" 
			if (top < 0) then 
				setY = setY - top 
			end 
		else 
			setY = -setY 
			bottom = bottom + setY 
			anchorPoint = "BOTTOM" 
			if (bottom > screenHeight) then 
				setY = setY + (screenHeight - bottom) 
			end 
		end 
		
		if (x <= (screenWidth * (1/2))) then 
			left = left + setX 
			if (anchorPoint == "BOTTOM") then 
				anchorPoint = anchorPoint.."RIGHT" 
				setX = setX - width 
				if (left < 0) then 
					setX = setX - left 
				end 
			else 
				anchorPoint = anchorPoint.."LEFT" 
				if (left < 0) then 
					setX = setX - left 
				end 
			end 
		else 
			setX = -setX 
			right = right + setX 
			if (anchorPoint == "BOTTOM") then 
				anchorPoint = anchorPoint.."LEFT" 
				setX = setX + width 
				if (right > screenWidth) then 
					setX = setX - (right - screenWidth) 
				end 
			else 
				anchorPoint = anchorPoint.."RIGHT" 
				if (right > screenWidth) then 
					setX = setX + (screenWidth - right) 
				end 
			end 
		end 
		
		scale = tooltip:GetScale() 
		if (scale) then 
			setX = setX / scale 
			setY = setY / scale 
		end
		
		tooltip:SetOwner(owner, "ANCHOR_"..anchorPoint, setX, setY)
	end
end

-- mostly stolen from PerfectTooltip by cladhaire
local Original_GameTooltip_SetDefaultAnchor
local function SetDefaultAnchor(tooltip,owner)
	if owner ~= UIParent or not uAnchorCursor then
		if uAnchorCursor == 2 then
			SmartSetOwner(owner, 0, 0, tooltip)
		else
			Original_GameTooltip_SetDefaultAnchor(tooltip,owner)
		end
	elseif tooltip == GameTooltip or tooltip == MobileGameTooltip then
		tooltip:ClearAllPoints()
		tooltip:SetOwner(owner, "ANCHOR_CURSOR")
	end
end

local Original_SetUnit
local function SetUnit(this,unit)
	Original_SetUnit(this,unit)
	tooltip_format(unit)
end

local function OnEvent()
	if event == "UPDATE_MOUSEOVER_UNIT" then
		if uScale then GameTooltip:SetScale(uScale) end
		if UnitExists("mouseover") then
            GameTooltip:SetUnit("mouseover")
            tooltip_format("mouseover")
		elseif uFade then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	end
end
-----------------------------------------------------------------------
-- Ready the mod
--
local EventFrame -- This is the frame we'll use for events.
local lastUpdate
local function OnUpdate() -- Hook here
	if not lastUpdate then 
		lastUpdate = arg1
	else
		if lastUpdate + 1 > arg1 then	
			Original_SetUnit = GameTooltip.SetUnit
			GameTooltip.SetUnit = SetUnit
			if uAnchorCursor then
				Original_GameTooltip_SetDefaultAnchor = GameTooltip_SetDefaultAnchor
				GameTooltip_SetDefaultAnchor = SetDefaultAnchor
			else
				SetDefaultAnchor = nil
			end -- OK, we're done with this
			EventFrame:SetScript("OnUpdate", nil)
			lastUpdate = nil
			OnUpdate = nil
		end
	end
end
		
local function tt_Initialize()
	EventFrame = CreateFrame("Frame", nil, UIParent)
	EventFrame:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	EventFrame:SetScript("OnEvent", OnEvent)
	EventFrame:SetScript("OnUpdate", OnUpdate)
end

tt_Initialize()
